@@ -17,7 +17,7 @@
  *  License along with this library; if not, write to the Free
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: shout.c 10687 2006-01-03 18:56:12Z brendan $
+ * $Id: shout.c 11554 2006-06-09 23:09:33Z brendan $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -162,7 +162,7 @@ int shout_close(shout_t *self)
 	if (self->state == SHOUT_STATE_UNCONNECTED)
 		return self->error = SHOUTERR_UNCONNECTED;
 
-	if (self->close)
+	if (self->state == SHOUT_STATE_CONNECTED && self->close)
 		self->close(self);
 
 	sock_close(self->socket);
@@ -312,6 +312,7 @@ int shout_set_metadata(shout_t *self, shout_metadata_t *metadata)
 
 		rv = sock_write(socket, "GET /admin/metadata?mode=updinfo&mount=%s&%s HTTP/1.0\r\nUser-Agent: %s\r\n%s\r\n",
 		  self->mount, encvalue, shout_get_agent(self), auth ? auth : "");
+                free(auth);
 	} else
 		rv = sock_write(socket, "GET /admin.cgi?mode=updinfo&pass=%s&mount=%s&%s HTTP/1.0\r\nUser-Agent: %s\r\n\r\n",
 		  self->password, self->mount, encvalue, shout_get_agent(self));
@@ -949,13 +950,14 @@ static int try_connect (shout_t *self)
 	case SHOUT_STATE_CONNECT_PENDING:
 		if (shout_get_nonblocking(self)) {
 			if ((rc = sock_connected(self->socket, 0)) < 1) {
-				if (!rc)
+				if (rc == SOCK_ERROR) {
+                                        rc = SHOUTERR_SOCKET;
+                                        goto failure;
+				} else
 					return SHOUTERR_BUSY;
-				else
-					return SHOUTERR_SOCKET;
 			}
 			if ((rc = create_request(self)) != SHOUTERR_SUCCESS)
-				return rc;
+                                goto failure;
 		}
 		self->state = SHOUT_STATE_REQ_PENDING;
 
@@ -963,36 +965,34 @@ static int try_connect (shout_t *self)
 		do
 			rc = send_queue(self);
 		while (!shout_get_nonblocking(self) && rc == SHOUTERR_BUSY);
+                if (rc == SHOUTERR_BUSY)
+                        return rc;
 		if (rc != SHOUTERR_SUCCESS)
-			return rc;
+                        goto failure;
 		self->state = SHOUT_STATE_RESP_PENDING;
 
 	case SHOUT_STATE_RESP_PENDING:
 		do
 			rc = get_response(self);
 		while (!shout_get_nonblocking(self) && rc == SHOUTERR_BUSY);
+                if (rc == SHOUTERR_BUSY)
+                        return rc;
+
 		if (rc != SHOUTERR_SUCCESS)
-			return rc;
+                        goto failure;
 
 		if ((rc = parse_response(self)) != SHOUTERR_SUCCESS)
-			return rc;
+                        goto failure;
 
 		if (self->format == SHOUT_FORMAT_OGG) {
-			if ((self->error = shout_open_ogg(self)) != SHOUTERR_SUCCESS) {
-				self->state = SHOUT_STATE_UNCONNECTED;
-				sock_close(self->socket);
-				return self->error;
-			}
+			if ((rc = self->error = shout_open_ogg(self)) != SHOUTERR_SUCCESS)
+                                goto failure;
 		} else if (self->format == SHOUT_FORMAT_MP3) {
-			if ((self->error = shout_open_mp3(self)) != SHOUTERR_SUCCESS) {
-				self->state = SHOUT_STATE_UNCONNECTED;
-				sock_close(self->socket);
-				return self->error;
-			}
+			if ((rc = self->error = shout_open_mp3(self)) != SHOUTERR_SUCCESS)
+                                goto failure;
 		} else {
-			self->state = SHOUT_STATE_UNCONNECTED;
-			sock_close(self->socket);
-			return self->error = SHOUTERR_INSANE;
+                        rc = SHOUTERR_INSANE;
+                        goto failure;
 		}
 
 	case SHOUT_STATE_CONNECTED:
@@ -1000,6 +1000,10 @@ static int try_connect (shout_t *self)
 	}
 	
 	return SHOUTERR_SUCCESS;
+
+failure:
+        shout_close(self);
+	return rc;
 }
 
 static int try_write (shout_t *self, const void *data, size_t len)
